﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Text;

namespace WebClientDemo
{
    internal sealed class MyWebClient : WebClient
    {
        private int _timeout = 30000;

        private static string _userAgent;

        private static string _referer;

        public int Timeout
        {
            get
            {
                return this._timeout;
            }
        }

        public WebProxy CustomWebProxy
        {
            get;
            set;
        }

        internal static string Referer
        {
            get
            {
                if (string.IsNullOrEmpty(MyWebClient._referer))
                {
                    MyWebClient._referer = "Test" + Environment.OSVersion.VersionString;
                }
                return MyWebClient._referer;
            }
        }

        internal static string UserAgent
        {
            get
            {
                if (string.IsNullOrEmpty(MyWebClient._userAgent))
                {
                    MyWebClient._userAgent = string.Concat(new object[]
                    {
                        "Test/",
                        typeof(MyWebClient).Assembly.GetName().Version,
                        "(",
                        Environment.OSVersion.VersionString,
                        ")"
                    });
                }
                return MyWebClient._userAgent;
            }
        }

        protected override WebRequest GetWebRequest(Uri address)
        {
            WebRequest webRequest = base.GetWebRequest(address);
            HttpWebRequest httpWebRequest = webRequest as HttpWebRequest;
            if (httpWebRequest != null)
            {
                httpWebRequest.AutomaticDecompression = (DecompressionMethods.GZip | DecompressionMethods.Deflate);
                httpWebRequest.Timeout = this.Timeout;
                httpWebRequest.UserAgent = MyWebClient.UserAgent;
                httpWebRequest.Referer = MyWebClient.Referer;
                httpWebRequest.CachePolicy = new System.Net.Cache.RequestCachePolicy(System.Net.Cache.RequestCacheLevel.Reload);
            }
            return webRequest;
        }

        protected override WebResponse GetWebResponse(WebRequest request)
        {
            if (request == null)
            {
                throw new ArgumentNullException("request");
            }
            Uri requestUri = request.RequestUri;

            if (CustomWebProxy != null)
            {
                request.Proxy = CustomWebProxy;
                WebResponse webResponse = MyWebClient.SubmitWebRequest(request);
                if (webResponse != null)
                {
                    return webResponse;
                }
            }
            if (WebRequest.DefaultWebProxy != null)
            {
                request.Proxy = WebRequest.DefaultWebProxy;
                request.Proxy.Credentials = CredentialCache.DefaultCredentials;
                WebResponse webResponse = MyWebClient.SubmitWebRequest(request);
                if (webResponse != null)
                {
                    return webResponse;
                }
            }

            return this.GetWebRequest(requestUri).GetResponse();
        }

        public static bool TryCreateUrlFromPartialAddress(string address, out Uri url)
        {
            address = address.Trim();
            if (string.IsNullOrEmpty(address))
            {
                url = null;
                return false;
            }
            try
            {
                if (address.StartsWith("http://", StringComparison.OrdinalIgnoreCase) || address.StartsWith("https://", StringComparison.OrdinalIgnoreCase))
                {
                    url = new Uri(address);
                }
                else if (address.StartsWith("//", StringComparison.Ordinal))
                {
                    url = new Uri("http:" + address);
                }
                else
                {
                    url = new Uri("http://" + address);
                }
                return true;
            }
            catch (UriFormatException)
            {
                url = null;
            }
            return false;
        }

        internal static WebResponse SubmitWebRequest(WebRequest request)
        {
            try
            {
                return request.GetResponse();
            }
            catch (WebException ex)
            {
                if (ex.Status == WebExceptionStatus.ProtocolError)
                {
                    throw;
                }
            }
            return null;
        }
    }
}
